@@ -5,8 +5,9 @@ source "$TOP/tests/common"
 source "$TOP/tests/common.convert"
 
 setup_root_helper
-prepare_test_dev 512M
+prepare_test_dev
 check_prereq btrfs-convert
+check_global_prereq mke2fs
 
 for bs in 1024 2048; do
 	default_mke2fs="mke2fs -t ext4 -b $bs"
